/*
 * _150453_circular_christmas_tree.c
 *
 * Created: 21/09/2015 09:18:52
 *  Author: CPV
 */ 

/*
ATmega328 Fuses:

BODLEVEL = 2V7
RSTDISBL = [ ]
DWEN = [ ]
SPIEN = [X]
WDTON = [ ]
EESAVE = [ ]
BOOTSZ = 256W_3F00
BOOTRST = [ ]
CKDIV8 = [ ]
CKOUT = [ ]
SUT_CKSEL = EXTFSXTAL_1KCK_14CK_0MS

EXTENDED = 0xFD (valid)
HIGH = 0xDF (valid)
LOW = 0xE6 (valid)
*/

#include "Elektor.h"
#include "systick.h"
#include "led_matrix.h"
#include "brightness.h"
#include "trimmer.h"
#include "running_led.h"


#define DO_TEST_AT_POWER_UP  (1)
#define TEST_WINDOW  (5)  /* sedonds */

#if DO_TEST_AT_POWER_UP==1
void main_test(void);

void main_test(void)
{
	uint16_t trimmer;
	// Disable running LEDs.
	running_led_enable(0);
	// All LEDs on.
	led_matrix_set_all();
	// Remain several seconds in test mode.
	while (get_seconds()<TEST_WINDOW)
	{
		// Trimmer adjusts brightness.
		trimmer = trimmer_read();
		brightness_set((uint8_t)(trimmer>>3)+1);
	}
	// All LEDs off.
	led_matrix_clear();
}
#endif


int main(void)
{
	uint32_t time;
	uint32_t time_prev;
	uint8_t b;
	uint16_t trimmer;
	uint32_t seconds_timeout;
	uint32_t seconds_prev;
		
	// Initialize system.	
	led_matrix_init();
	systick_init();
	brightness_init();
	trimmer_init();
	srand(trimmer_read());
	running_led_init();
	b = 0x01;
	brightness_set(BRIGHTNESS_MIN);
	
	// Enable interrupts.
	sei();

	time_prev = get_milliseconds();
	seconds_timeout = 0;
	
#if DO_TEST_AT_POWER_UP==1
	main_test();
#endif

	running_led_enable(1);
	
    while (1)
    {
		trimmer = trimmer_read();

		time = get_milliseconds();
		if (time>time_prev+50)
		{
			time_prev = time;

			// Run 50 ms tasks.

			// Update LED timeout timers.
			led_matrix_tick_timeout();

			// Update brightness.			
			// Exponential values for brightness are nicer than linear.
			// This is why we shift instead of count.
			/*if (b==0)
			{
				brightness <<= 1;
				if (brightness==0)
				{
					b = 1;
					brightness = 0x80;
				}
			}
			else
			{
				brightness >>= 1;
				if (brightness==0)
				{
					b = 0;
					brightness = 0x01;
				}
			}*/
			//brightness = trimmer>>3;
			//brightness_set((uint8_t)(trimmer>>3)+1);
		}

		// One second tasks.
		if (get_seconds()>seconds_prev)
		{
			seconds_prev = get_seconds();
			// Every second do:
			// Switch a pixel off to prevent pixels from staying on.
			led_matrix_put_pixel(b,black);
			// Next pixel;
			b += 1;
			if (b>=LEDS) b = 0;
		}
		
		// Slow tasks.
		if (get_seconds()>=seconds_timeout)
		{
			seconds_timeout = get_seconds() + rand()%5 + (trimmer>>7) + 2;
			// Every 15 seconds do:
			// Activate a randomly selected ring with random timeout values.
			led_matrix_ring_timeout_random(rand()%RINGS,white,0);
		}
    }
}
