/*
 * led_matrix.c
 *
 * Created: 21/09/2015 09:52:03
 *  Author: CPV
 */ 

#include "led_matrix.h"
#include "brightness.h"


// Book-keeping table for the LEDs.
volatile led_matrix_led_t leds[LEDS] =
{
	// row    column  color  timer
	{ PORTD2, PORTB0, black, 0, 0 }, // Ring 0, Row 0, Column 0, LED1
	{ PORTD2, PORTB1, black, 0, 0 }, // Ring 0, Row 0, Column 1, LED2
	{ PORTD2, PORTB2, black, 0, 0 }, // Ring 0, Row 0, Column 2, LED3
	{ PORTD2, PORTB3, black, 0, 0 }, // Ring 0, Row 0, Column 3, LED4
	{ PORTD2, PORTB4, black, 0, 0 }, // Ring 0, Row 0, Column 4, LED5
	{ PORTD2, PORTB5, black, 0, 0 }, // Ring 0, Row 0, Column 5, LED6
	{ PORTD3, PORTB0, black, 0, 0 }, // Ring 0, Row 1, Column 0, LED7
	{ PORTD3, PORTB1, black, 0, 0 }, // Ring 0, Row 1, Column 1, LED8
	{ PORTD3, PORTB2, black, 0, 0 }, // Ring 0, Row 1, Column 2, LED9
	{ PORTD3, PORTB3, black, 0, 0 }, // Ring 0, Row 1, Column 3, LED10
	{ PORTD3, PORTB4, black, 0, 0 }, // Ring 0, Row 1, Column 4, LED11
	{ PORTD3, PORTB5, black, 0, 0 }, // Ring 1, Row 1, Column 5, LED12
	{ PORTD4, PORTB0, black, 0, 0 }, // Ring 1, Row 2, Column 0, LED13
	{ PORTD4, PORTB1, black, 0, 0 }, // Ring 1, Row 2, Column 1, LED14
	{ PORTD4, PORTB2, black, 0, 0 }, // Ring 1, Row 2, Column 2, LED15
	{ PORTD4, PORTB3, black, 0, 0 }, // Ring 1, Row 2, Column 3, LED16
	{ PORTD4, PORTB4, black, 0, 0 }, // Ring 1, Row 2, Column 4, LED17
	{ PORTD4, PORTB5, black, 0, 0 }, // Ring 1, Row 2, Column 5, LED18
	{ PORTD5, PORTB0, black, 0, 0 }, // Ring 1, Row 3, Column 0, LED19
	{ PORTD5, PORTB1, black, 0, 0 }, // Ring 1, Row 3, Column 1, LED20
	{ PORTD5, PORTB2, black, 0, 0 }, // Ring 2, Row 3, Column 2, LED21
	{ PORTD5, PORTB3, black, 0, 0 }, // Ring 2, Row 3, Column 3, LED22
	{ PORTD5, PORTB4, black, 0, 0 }, // Ring 2, Row 3, Column 4, LED23
	{ PORTD5, PORTB5, black, 0, 0 }, // Ring 2, Row 3, Column 5, LED24
	{ PORTD6, PORTB0, black, 0, 0 }, // Ring 2, Row 4, Column 0, LED25
	{ PORTD6, PORTB1, black, 0, 0 }, // Ring 2, Row 4, Column 1, LED26
	{ PORTD6, PORTB2, black, 0, 0 }, // Ring 2, Row 4, Column 2, LED27
	{ PORTD6, PORTB3, black, 0, 0 }, // Ring 3, Row 4, Column 3, LED28
	{ PORTD6, PORTB4, black, 0, 0 }, // Ring 3, Row 4, Column 4, LED29
	{ PORTD6, PORTB5, black, 0, 0 }, // Ring 3, Row 4, Column 5, LED30
	{ PORTD7, PORTB0, black, 0, 0 }, // Ring 3, Row 5, Column 0, LED31
	{ PORTD7, PORTB1, black, 0, 0 }, // Ring 3, Row 5, Column 1, LED32
	{ PORTD7, PORTB2, black, 0, 0 }, // Ring 4, Row 5, Column 2, LED33
	{ PORTD7, PORTB3, black, 0, 0 }, // Ring 4, Row 5, Column 3, LED34
	{ PORTD7, PORTB4, black, 0, 0 }, // Ring 4, Row 5, Column 4, LED35
	{ PORTD7, PORTB5, black, 0, 0 }, // Ring 5, Row 5, Column 5, LED36
};

// This is the number of the LED that is actually being driven at any time.
// All the others are disconnected.
static uint8_t led_active = 0;

// Ring look-up table.
uint8_t number_of_leds_on_ring[RINGS] =
{
	11, 9, 7, 5, 3, 1
};

// Ring look-up table.
uint8_t ring_leds[RINGS][LEDS_PER_RING_MAX] =
{
	{  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10 }, // LEDs 1-11
	{ 11, 12, 13, 14, 15, 16, 17, 18, 19, -1, -1 }, // LEDs 12-20 (-1 indicates no LED)
	{ 20, 21, 22, 23, 24, 25, 26, -1, -1, -1, -1 }, // LEDs 21-27
	{ 27, 28, 29, 30, 31, -1, -1, -1, -1, -1, -1 }, // LEDs 28-32
	{ 32, 33, 34, -1, -1, -1, -1, -1, -1, -1, -1 }, // LEDs 33-35
	{ 35, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1 }, // LED 36
};


void led_matrix_init(void)
{
	int i;

	MCUCR |= (1<<PUD); // Globally disable internal pull-up resistors.
	DDRD = 0xfc; // PD2-7 outputs.
	DDRB = 0x3f; // PB0-5 outputs.
	
	for (i=0; i<LEDS; i++)
	{
		// All LEDs off.
		led_matrix_set(i,black);
	}

	led_active = 0;
}


void led_matrix_set(uint8_t nr, led_matrix_color_t color)
{
	switch (color)
	{
		case black:
		// Set pins to input.
		ROW_PORT_DDR &= ~(1<<leds[nr].row);
		COL_PORT_DDR &= ~(1<<leds[nr].column);
		// Set output levels.
		break;

		//case white:
		default:
		// Set pins to output.
		ROW_PORT_DDR |= (1<<leds[nr].row);
		COL_PORT_DDR |= (1<<leds[nr].column);
		// Set output levels.
		ROW_PORT &= ~(1<<leds[nr].row); // 0 (cathode)
		COL_PORT |= (1<<leds[nr].column); // 1 (anode)
		break;
	}
}


void led_matrix_disconnect(void)
{
	// Only one LED is active at any time, so to disconnect the matrix
	// it is enough to deactivate this LED.
	ROW_PORT_DDR &= ~(1<<leds[led_active].row);
	COL_PORT_DDR &= ~(1<<leds[led_active].column);
}


void led_matrix_connect(void)
{
	// Reactivate current active LED.
	led_matrix_set(led_active,leds[led_active].color); 
}


void led_matrix_put_pixel(uint8_t nr, led_matrix_color_t color)
{
	static uint16_t b = 0x01;
	
	// We use usage counts to allow different "threads" use pixels as if they
	// were there own. Only when the usage count falls to zero then a pixel will go off.
	if (nr<LEDS)
	{
		// If pixel is on, increase usage count.
		// Else decrease usage count.
		if (color==white)
		{
			if (leds[nr].color<2) leds[nr].color += 1;
			else leds[nr].color = 2;
			if (leds[nr].color==1)
			{
				leds[nr].brightness = b&0xff;
				b <<= 1;
				if (b>0x100) b = 0x01;
			}
		}
		else if (leds[nr].color>0) leds[nr].color -= 1;
	}
}


void led_matrix_set_timeout(uint8_t nr, uint8_t timeout)
{
	if (nr<LEDS)
	{
		leds[nr].timer = timeout;
	}
}


void led_matrix_put_pixel_timeout(uint8_t nr, led_matrix_color_t color, uint8_t timeout)
{
	if (nr<LEDS && leds[nr].timer==0)
	{
		led_matrix_put_pixel(nr,color);
		led_matrix_set_timeout(nr,timeout);
	}
}


led_matrix_color_t led_matrix_get_pixel(uint8_t nr)
{
	if (nr<LEDS)
	{
		return leds[nr].color>0? white : black;
	}
	return black;
}


void led_matrix_tick(void)
{
	// This must be called often to ensure a flicker-free display.
	// 50 Hz times the number of LEDs is a good minimum.
	uint8_t led_max = LEDS;
	led_matrix_set(led_active,black); // Deactivate current LED.
	led_active += 1; // Increment LED counter.
	if (led_active>=led_max) led_active = 0; // Wrap around if needed.
	led_matrix_set(led_active,leds[led_active].color); // Activate new LED.
	brightness_set(leds[led_active].brightness);
}


void led_matrix_tick_timeout(void)
{
	int i;

	// Decrement timeout counters.
	for (i=0; i<LEDS; i++)
	{
		if (leds[i].timer>0)
		{
			leds[i].timer -= 1;
			if (leds[i].timer==0)
			{
				led_matrix_put_pixel(i,black);
			}
		}
	}
}


void led_matrix_clear(void)
{
	int i;

	for (i=0; i<LEDS; i++)
	{
		led_matrix_put_pixel(i,black);
	}
}


void led_matrix_set_all(void)
{
	int i;

	for (i=0; i<LEDS; i++)
	{
		leds[i].color = 1;
		leds[i].brightness = 0;
		leds[i].timer = 0;
	}
}


uint8_t led_matrix_led_to_ring(uint8_t nr)
{
	if (nr<11) return 0; // 1-11
	else if (nr<20) return 1; // 12-20
	else if (nr<27) return 2; // 21-27
	else if (nr<32) return 3; // 28-32
	else if (nr<35) return 4; // 33-35
	else return 5; // 36
}


uint8_t led_matrix_number_of_leds_on_ring(uint8_t nr)
{
	if (nr<RINGS) return number_of_leds_on_ring[nr];
	else return (uint8_t)-1;
}


void led_matrix_ring(uint8_t ring, led_matrix_color_t color)
{
	uint8_t i;
	uint8_t led_nr;
	for (i=0; i<LEDS_PER_RING_MAX; i++)
	{
		led_nr = ring_leds[ring][i];
		if (led_nr==(uint8_t)-1) return;
		led_matrix_put_pixel(led_nr,color);
	}
}


void led_matrix_ring_timeout(uint8_t ring, led_matrix_color_t color, uint8_t timeout)
{
	uint8_t i;
	uint8_t led_nr;
	for (i=0; i<LEDS_PER_RING_MAX; i++)
	{
		led_nr = ring_leds[ring][i];
		if (led_nr==(uint8_t)-1) return;
		led_matrix_put_pixel_timeout(led_nr,color,timeout);
	}
}


void led_matrix_ring_timeout_random(uint8_t ring, led_matrix_color_t color, uint8_t timeout_min)
{
	uint8_t i;
	uint8_t led_nr;
	for (i=0; i<LEDS_PER_RING_MAX; i++)
	{
		led_nr = ring_leds[ring][i];
		if (led_nr==(uint8_t)-1) return;
		led_matrix_put_pixel_timeout(led_nr,color,(rand()&0xff)+timeout_min); // 8-bit timeout may overflow, so be careful.
	}
}
