/*
 * running_led.c
 *
 * Created: 27/09/2015 10:09:33
 *  Author: CPV
 */ 


/*
This code handles multiple threads of running LEDs. A running LED is an LED that moves
around the matrix along a predefined path. It has several parameters which allow it to 
start with a delay, accelerate, decelerate, run clockwise or anti-clockwise, etc. All
these parameters can be changed on-the-fly.

A running LED initial parameters are stored in a table.
*/


#include "running_led.h"


volatile running_led_t running_led[RUNNING_LEDS] =
{
	{    1, 200,  35, 40,  1, 36, white, 250, 0, 0, 0 },
	{ 1250, 200,  35, 40, -1, 36, white, 250, 0, 0, 0 },
	{  250, 200,  35, 40,  1, 36, white, 250, 0, 0, 0 },
	{  500, 200, 100, 25, -1, 36, white, 250, 0, 0, 0 },
	{  666, 200,  89, 13,  1, 36, white, 250, 0, 0, 0 },
	{  777, 200,  77,  7, -1, 36, white, 250, 0, 0, 0 },
};

static uint8_t running_led_enabled;


void running_led_enable(uint8_t value)
{
	running_led_enabled = value;
}


void running_led_tick(running_led_t *p_led)
{
	if (running_led_enabled==0) return;
	
	// Called from systick ISR @ 1 kHz.
	if (p_led->delay>0)
	{
		p_led->delay -= 1;
		if (p_led->delay==0)
		{
			p_led->speed = p_led->speed_min;
			p_led->steps_taken = 0;
		}
	}
	else if (p_led->steps_taken<p_led->steps_to_take)
	{
		p_led->life++;
		if (p_led->life>=p_led->speed)
		{
			// Reset speed counter.
			p_led->life = 0;

			// Move the LED.
			led_matrix_put_pixel(p_led->position,black);
			p_led->position = (p_led->position+p_led->step);
			if (p_led->position>=LEDS) p_led->position = 0;
			else if (p_led->position<0) p_led->position = LEDS-1;
			p_led->steps_taken += abs(p_led->step);
			led_matrix_put_pixel(p_led->position,p_led->color);

			// Update speed.
			if (p_led->steps_taken<=(p_led->steps_to_take>>1)+2)
			{
				// Accelerate.
				if (p_led->speed>p_led->speed_max)
				{
					p_led->speed -= p_led->acceleration;
					if (p_led->speed<p_led->speed_max)
					{
						p_led->speed = p_led->speed_max;
					}
				}
			}
			else
			{
				// Decelerate.
				if (p_led->speed<p_led->speed_min)
				{
					p_led->speed += p_led->acceleration;
					if (p_led->speed>p_led->speed_min)
					{
						p_led->speed = p_led->speed_min;
					}
				}
			}
		}
	}
	else
	{
		p_led->delay = 1000;
	}
}


void running_led_tick_all(void)
{
	uint8_t i;
	for (i=0; i<RUNNING_LEDS; i++)
	{
		running_led_tick((running_led_t*)&running_led[i]);
	}
}


void running_led_init(void)
{
	int i;
	for (i=0; i<RUNNING_LEDS; i++)
	{
		running_led[i].speed = running_led[i].speed_min;
		running_led[i].steps_taken = 0;
		running_led[i].life = 0;
		led_matrix_put_pixel(running_led[i].position,running_led[i].color);
	}
	running_led_enabled = 0;
}

