/*
 * systick.c
 *
 * Created: 27/09/2015 10:29:21
 *  Author: CPV
 */ 

#include "systick.h"
#include "running_led.h"
#include "brightness.h"


volatile uint32_t systicks = 0;
volatile uint32_t milliseconds = 0;
volatile uint32_t seconds = 0;


ISR(TIMER0_COMPA_vect)
{
	// Systick interrupt.
	
	systicks += 1;
	if (systicks%5==0)
	{
		// Update millisecond counter.
		milliseconds++;
		if (milliseconds%1000==0)
		{
			seconds += 1;
		}
		
		// Run millisecond tasks.
		running_led_tick_all();
	}
	
	// Refresh LED matrix.
	led_matrix_tick();
	
	// Launch brightness timer.
	TCNT2 = 0;
	// Clear & enable brightness interrupt.
	TIFR2 |= (1<<OCF2A);
	TIMSK2 |= (1<<OCIE2A);
}


void systick_init(void)
{
	systicks = 0;
	milliseconds = 0;
	seconds = 0;
	// CTC mode (WGM0 = 0x02), Compare Match Outputs disabled (enable for debugging only).
	TCCR0A = (1<<WGM01) /*| (1<<COM0A0) | (1<<COM0B0)*/;
	// Prescaler = 64 (CS0 = 0x03)
	TCCR0B = (1<<CS01) | (1<<CS00);
	// Use Output Compare A: 16 MHz / 64 / (49+1) = 5000 Hz
	// With 36 LEDs this results in a refresh rate of almost 140 Hz.
	OCR0A = 49;
	// Enable interrupt on OCR0A
	TIMSK0 = (1<<OCIE0A);
}


uint32_t get_milliseconds(void)
{
	uint32_t result;
	// Disable interrupts to avoid reading invalid values.
	cli();
	result = milliseconds;
	// Re-enable interrupts.
	sei();
	return result;
}


uint32_t get_seconds(void)
{
	uint32_t result;
	// Disable interrupts to avoid reading invalid values.
	cli();
	result = seconds;
	// Re-enable interrupts.
	sei();
	return result;
}
